# ALMA Data Reduction Script

# Notes:

# - No tsys or wvr problems noted
# - Flux cal amplitude updated on all spws
# - Bandpass command was updated to solint='inf,15.6Mhz',field='',intent='BANDPASS'
# - 3 out of 33 antennas completely flagged or missing: DA62, DA49 and DV07
# DV07: large amplitude roll-off in bandpass, large delay in phase solution
#       Also, low amplitude gains on all spws
# DA62: No data
# DV06: No data on spws 3~6
# DV07: quite a bit of decorrelation in phase gain solutions for spw3~6
# DA49 and DV07: larger phase rms and amplitude rms after calibration, flagged

# Calibration

thesteps = []
step_title = {0: 'Import of the ASDM',
              1: 'Fix of SYSCAL table times',
              2: 'listobs',
              3: 'A priori flagging',
              4: 'Generation and time averaging of the WVR cal table',
              5: 'Generation of the Tsys cal table',
              6: 'Generation of the antenna position cal table',
              7: 'Application of the WVR, Tsys and antpos cal tables',
              8: 'Split out science SPWs and time average',
              9: 'Listobs, clear pointing table, and save original flags',
              10: 'Initial flagging',
              11: 'Putting a model for the flux calibrator(s)',
              12: 'Save flags before bandpass cal',
              13: 'Bandpass calibration',
              14: 'Save flags before gain cal',
              15: 'Gain calibration',
              16: 'Save flags before applycal',
              17: 'Application of the bandpass and gain cal tables',
              18: 'Split out corrected column'}

if 'applyonly' not in globals(): applyonly = False
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

import re

import os

if applyonly != True: es = aU.stuffForScienceDataReduction() 


if re.search('^4.2.2', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 4.2.2')


# CALIBRATE_AMPLI: J0334-4008
# CALIBRATE_ATMOSPHERE: J0215-0222,J0241-0815,J0334-4008,mira
# CALIBRATE_BANDPASS: J0241-0815
# CALIBRATE_FLUX: J0334-4008
# CALIBRATE_FOCUS: 
# CALIBRATE_PHASE: J0215-0222
# CALIBRATE_POINTING: J0215-0222,J0241-0815,J0334-4008
# OBSERVE_TARGET: mira

# Using reference antenna = DV04

# Import of the ASDM
mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  if os.path.exists('uid___A002_X905825_X5a8.ms') == False:
    importasdm('uid___A002_X905825_X5a8', asis='Antenna Station Receiver Source CalAtmosphere CalWVR')
  if applyonly != True: es.fixForCSV2555('uid___A002_X905825_X5a8.ms')

# Fix of SYSCAL table times
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  from recipes.almahelpers import fixsyscaltimes
  fixsyscaltimes(vis = 'uid___A002_X905825_X5a8.ms')

print "# A priori calibration"

# listobs
mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X905825_X5a8.ms.listobs')
  listobs(vis = 'uid___A002_X905825_X5a8.ms',
    listfile = 'uid___A002_X905825_X5a8.ms.listobs')
  
  

# A priori flagging
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  flagdata(vis = 'uid___A002_X905825_X5a8.ms',
    mode = 'manual',
    spw = '1~27',
    autocorr = T,
    flagbackup = F)
  
  flagdata(vis = 'uid___A002_X905825_X5a8.ms',
    mode = 'manual',
    intent = '*POINTING*,*ATMOSPHERE*',
    flagbackup = F)
  
  flagcmd(vis = 'uid___A002_X905825_X5a8.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'plot',
    plotfile = 'uid___A002_X905825_X5a8.ms.flagcmd.png')
  
  flagcmd(vis = 'uid___A002_X905825_X5a8.ms',
    inpmode = 'table',
    useapplied = True,
    action = 'apply')
  

# Generation and time averaging of the WVR cal table
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X905825_X5a8.ms.wvr') 
  
  os.system('rm -rf uid___A002_X905825_X5a8.ms.wvrgcal') 
  
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X905825_X5a8.ms.wvrgcal')
  
  wvrgcal(vis = 'uid___A002_X905825_X5a8.ms',
    caltable = 'uid___A002_X905825_X5a8.ms.wvr',
    toffset = 0,
    tie = ['mira,J0215-0222'],
    statsource = 'mira')
  
  casalog.setlogfile(mylogfile)
  
  # This is a temporary workaround, which will be included in a future version of CASA
  
  tb.open('uid___A002_X905825_X5a8.ms.wvr', nomodify=False)
  count = 0
  numrows = tb.nrows()
  mycparamcol = tb.getcol('CPARAM')
  for i in range(0, numrows):
      if mycparamcol[0][0][i] == (1.+0.j):
          tb.putcell('FLAG', i, [[True]])
          count += 1
  tb.close()
  del mycparamcol
  if(numrows>0):
      print 'Flagged', count, 'of', numrows, 'solutions =', 100.*count/float(numrows),'%'
  
  
  os.system('rm -rf uid___A002_X905825_X5a8.ms.wvr.smooth') 
  
  smoothcal(vis = 'uid___A002_X905825_X5a8.ms',
    tablein = 'uid___A002_X905825_X5a8.ms.wvr',
    caltable = 'uid___A002_X905825_X5a8.ms.wvr.smooth',
    smoothtype = 'mean',
    smoothtime = 6.048)
  
  
  if applyonly != True: aU.plotWVRSolutions(caltable='uid___A002_X905825_X5a8.ms.wvr.smooth', spw='11', antenna='DV04',
    yrange=[-199,199],subplot=22, interactive=False,
    figfile='uid___A002_X905825_X5a8.ms.wvr.smooth.plots/uid___A002_X905825_X5a8.ms.wvr.smooth') 
  
  #Note: If you see wraps in these plots, try changing yrange or unwrap=True 
  #Note: If all plots look strange, it may be a bad WVR on the reference antenna.
  #      To check, you can set antenna='' to show all baselines.
  

# Generation of the Tsys cal table
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X905825_X5a8.ms.tsys') 
  gencal(vis = 'uid___A002_X905825_X5a8.ms',
    caltable = 'uid___A002_X905825_X5a8.ms.tsys',
    caltype = 'tsys')
  
  if applyonly != True: aU.plotbandpass(caltable='uid___A002_X905825_X5a8.ms.tsys', overlay='time', 
    xaxis='freq', yaxis='amp', subplot=22, buildpdf=False, interactive=False,
    showatm=True,pwv='auto',chanrange='5~123',showfdm=True, 
    field='', figfile='uid___A002_X905825_X5a8.ms.tsys.plots.overlayTime/uid___A002_X905825_X5a8.ms.tsys') 
  
  
  if applyonly != True: es.checkCalTable('uid___A002_X905825_X5a8.ms.tsys', msName='uid___A002_X905825_X5a8.ms', interactive=False) 
  

# Generation of the antenna position cal table
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Note: no baseline run found for antenna DV12.
  
  # Note: the correction for antenna DV19 is larger than 2mm.
  
  # Position for antenna DV19 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: the correction for antenna DA65 is larger than 2mm.
  
  # Position for antenna DA65 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: the correction for antenna DA64 is larger than 2mm.
  
  # Position for antenna DA64 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: the correction for antenna DA49 is larger than 2mm.
  
  # Position for antenna DA49 is derived from baseline run made on 2014-06-11 04:48:50.
  
  # Note: no baseline run found for antenna DA48.
  
  # Note: the correction for antenna DA61 is larger than 2mm.
  
  # Position for antenna DA61 is derived from baseline run made on 2014-07-20 05:52:41.
  
  # Note: the correction for antenna DA60 is larger than 2mm.
  
  # Position for antenna DA60 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: no baseline run found for antenna DA45.
  
  # Note: no baseline run found for antenna DV10.
  
  # Note: no baseline run found for antenna DV13.
  
  # Note: the correction for antenna DA46 is larger than 2mm.
  
  # Position for antenna DA46 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: no baseline run found for antenna DA41.
  
  # Note: no baseline run found for antenna DV14.
  
  # Note: no baseline run found for antenna DA43.
  
  # Position for antenna DA42 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Position for antenna DV25 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: no baseline run found for antenna DA63.
  
  # Note: no baseline run found for antenna DV15.
  
  # Note: no baseline run found for antenna DA62.
  
  # Note: no baseline run found for antenna DV09.
  
  # Note: no baseline run found for antenna DV11.
  
  # Note: the correction for antenna DV22 is larger than 2mm.
  
  # Position for antenna DV22 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: the correction for antenna DA52 is larger than 2mm.
  
  # Position for antenna DA52 is derived from baseline run made on 2012-11-15 10:54:37.
  
  # Note: no baseline run found for antenna DA50.
  
  # Position for antenna DA51 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: no baseline run found for antenna DA56.
  
  # Note: the correction for antenna DA57 is larger than 2mm.
  
  # Position for antenna DA57 is derived from baseline run made on 2014-09-05 03:49:41.
  
  # Note: no baseline run found for antenna DV08.
  
  # Note: the correction for antenna DA55 is larger than 2mm.
  
  # Position for antenna DA55 is derived from baseline run made on 2014-07-20 05:52:41.
  
  # Note: no baseline run found for antenna DV06.
  
  # Note: no baseline run found for antenna DV07.
  
  # Note: the correction for antenna DV04 is larger than 2mm.
  
  # Position for antenna DV04 is derived from baseline run made on 2014-09-05 03:49:41.
  
  os.system('rm -rf uid___A002_X905825_X5a8.ms.antpos') 
  gencal(vis = 'uid___A002_X905825_X5a8.ms',
    caltable = 'uid___A002_X905825_X5a8.ms.antpos',
    caltype = 'antpos',
    antenna = 'DV19,DA52,DA65,DA64,DA49,DV25,DA61,DA60,DV04,DA46,DV22,DA42,DA51,DA55,DA57',
    parameter = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0])
  #  parameter = [-0.00414290977642,0.0092498883605,0.00376623217016,0.000253285747021,0.00267631281167,0.00169306434691,-0.0019226311706,0.00491832382977,0.00211138557643,-0.00410456955433,0.00726661179215,0.00325340544805,-0.000611576717347,0.00173676759005,0.00119661167264,-0.000981642864645,0.00105158798397,0.000677143689245,-0.00103083346039,0.0023907283321,0.0013208379969,-0.00318624591455,0.00662772543728,0.00263020861894,-0.00236665643752,0.00325250066817,0.000717006158084,-0.00178120005876,0.00230418425053,0.000323298387229,-0.0017504398711,0.00266457349062,0.000778917223215,-0.000526665726542,0.000489968824986,0.000222492163967,-0.000559699778247,0.000879617942017,0.000171634595327,-0.000833045380458,0.00211589048263,0.00105222804544,-0.000978342723101,0.00163009297103,0.000727996230125])
  

# Application of the WVR, Tsys and antpos cal tables
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  
  from recipes.almahelpers import tsysspwmap
  tsysmap = tsysspwmap(vis = 'uid___A002_X905825_X5a8.ms', tsystable = 'uid___A002_X905825_X5a8.ms.tsys', tsysChanTol = 1)
  
  
  
  applycal(vis = 'uid___A002_X905825_X5a8.ms',
    field = '0',
    spw = '11,13,15,17,19,21,23',
    gaintable = ['uid___A002_X905825_X5a8.ms.tsys', 'uid___A002_X905825_X5a8.ms.wvr.smooth', 'uid___A002_X905825_X5a8.ms.antpos'],
    gainfield = ['0', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X905825_X5a8.ms',
    field = '1',
    spw = '11,13,15,17,19,21,23',
    gaintable = ['uid___A002_X905825_X5a8.ms.tsys', 'uid___A002_X905825_X5a8.ms.wvr.smooth', 'uid___A002_X905825_X5a8.ms.antpos'],
    gainfield = ['1', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X905825_X5a8.ms',
    field = '2',
    spw = '11,13,15,17,19,21,23',
    gaintable = ['uid___A002_X905825_X5a8.ms.tsys', 'uid___A002_X905825_X5a8.ms.wvr.smooth', 'uid___A002_X905825_X5a8.ms.antpos'],
    gainfield = ['2', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  
  
  applycal(vis = 'uid___A002_X905825_X5a8.ms',
    field = '3',
    spw = '11,13,15,17,19,21,23',
    gaintable = ['uid___A002_X905825_X5a8.ms.tsys', 'uid___A002_X905825_X5a8.ms.wvr.smooth', 'uid___A002_X905825_X5a8.ms.antpos'],
    gainfield = ['3', '', ''],
    interp = 'linear,linear',
    spwmap = [tsysmap,[],[]],
    calwt = T,
    flagbackup = F)
  
  if applyonly != True: es.getCalWeightStats('uid___A002_X905825_X5a8.ms') 
  

# Split out science SPWs and time average
mystep = 8
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X905825_X5a8.ms.split') 
  split(vis = 'uid___A002_X905825_X5a8.ms',
    outputvis = 'uid___A002_X905825_X5a8.ms.split',
    datacolumn = 'corrected',
    spw = '11,13,15,17,19,21,23',
    keepflags = T)
  
  

print "# Calibration"

# Listobs, clear pointing table, and save original flags
mystep = 9
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X905825_X5a8.ms.split.listobs')
  listobs(vis = 'uid___A002_X905825_X5a8.ms.split',
    listfile = 'uid___A002_X905825_X5a8.ms.split.listobs')
  
  tb.open('uid___A002_X905825_X5a8.ms.split/POINTING', nomodify = False)
  a = tb.rownumbers()
  tb.removerows(a)
  tb.close()
  
  if not os.path.exists('uid___A002_X905825_X5a8.ms.split.flagversions/Original.flags'):
    flagmanager(vis = 'uid___A002_X905825_X5a8.ms.split',
      mode = 'save',
      versionname = 'Original')
  
  

# Initial flagging
mystep = 10
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # Flagging shadowed data
  
  flagdata(vis = 'uid___A002_X905825_X5a8.ms.split',
    mode = 'shadow',
    flagbackup = F)
  
  # Flagging edge channels
  
  flagdata(vis = 'uid___A002_X905825_X5a8.ms.split',
    mode = 'manual',
    spw = '0:0~7;120~127,1:0~7;120~127,2:0~7;120~127',
    flagbackup = F)
  
  

# Putting a model for the flux calibrator(s)
mystep = 11
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  # au.getALMAFluxForMS('uid___A002_X905825_X5a8.ms.split',field='1',frequency='88.1854363613GHz')
  setjy('uid___A002_X905825_X5a8.ms.split',
      standard='manual', field='J0334-4008', spix=-0.714632,
      reffreq='88.185436GHz', fluxdensity=[1.636895,0,0,0])

"""
  setjy(vis = 'uid___A002_X905825_X5a8.ms.split',
    field = '1', # source name = J0334-4008
    spw = '0', # center frequency of spw = 88.1854363613GHz
    standard = 'manual',
    fluxdensity = [1.65896840988, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
  
  setjy(vis = 'uid___A002_X905825_X5a8.ms.split',
    field = '1', # source name = J0334-4008
    spw = '1', # center frequency of spw = 98.2274363613GHz
    standard = 'manual',
    fluxdensity = [1.65896840988, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
  
  setjy(vis = 'uid___A002_X905825_X5a8.ms.split',
    field = '1', # source name = J0334-4008
    spw = '2', # center frequency of spw = 100.185436361GHz
    standard = 'manual',
    fluxdensity = [1.65896840988, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
  
  setjy(vis = 'uid___A002_X905825_X5a8.ms.split',
    field = '1', # source name = J0334-4008
    spw = '3', # center frequency of spw = 86.8251536464GHz
    standard = 'manual',
    fluxdensity = [1.65896840988, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
  
  setjy(vis = 'uid___A002_X905825_X5a8.ms.split',
    field = '1', # source name = J0334-4008
    spw = '4', # center frequency of spw = 86.221668539GHz
    standard = 'manual',
    fluxdensity = [1.65896840988, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
  
  setjy(vis = 'uid___A002_X905825_X5a8.ms.split',
    field = '1', # source name = J0334-4008
    spw = '5', # center frequency of spw = 85.6188853359GHz
    standard = 'manual',
    fluxdensity = [1.65896840988, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
  
  setjy(vis = 'uid___A002_X905825_X5a8.ms.split',
    field = '1', # source name = J0334-4008
    spw = '6', # center frequency of spw = 85.7375987148GHz
    standard = 'manual',
    fluxdensity = [1.65896840988, 0, 0, 0]) # frequency of measurement = 86.22510445GHz
"""  
  

# Save flags before bandpass cal
mystep = 12
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X905825_X5a8.ms.split',
    mode = 'save',
    versionname = 'BeforeBandpassCalibration')
  
  

# Bandpass calibration
mystep = 13
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X905825_X5a8.ms.split.ap_pre_bandpass') 
  
  gaincal(vis = 'uid___A002_X905825_X5a8.ms.split',
    caltable = 'uid___A002_X905825_X5a8.ms.split.ap_pre_bandpass',
    field = '0', # J0241-0815
    spw = '0:51~76,1:51~76,2:51~76,3:384~576,4:384~576,5:384~576,6:384~576',
    solint = 'int',
    refant = 'DV04',
    calmode = 'p')
  
  if applyonly != True: es.checkCalTable('uid___A002_X905825_X5a8.ms.split.ap_pre_bandpass', msName='uid___A002_X905825_X5a8.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X905825_X5a8.ms.split.bandpass') 
  bandpass(vis = 'uid___A002_X905825_X5a8.ms.split',
    caltable = 'uid___A002_X905825_X5a8.ms.split.bandpass',
    field = '', # J0241-0815
    intent = '*BANDPASS*',
    solint = 'inf,0.85449MHz',
    combine = 'scan',
    refant = 'DV04',
    solnorm = True,
    bandtype = 'B',
    gaintable = 'uid___A002_X905825_X5a8.ms.split.ap_pre_bandpass')
  #    solint = 'inf,15.6MHz',

  if applyonly != True: es.checkCalTable('uid___A002_X905825_X5a8.ms.split.bandpass', msName='uid___A002_X905825_X5a8.ms.split', interactive=False) 
  

# Save flags before gain cal
mystep = 14
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X905825_X5a8.ms.split',
    mode = 'save',
    versionname = 'BeforeGainCalibration')
  
  

# Gain calibration
mystep = 15
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X905825_X5a8.ms.split.phase_int') 
  gaincal(vis = 'uid___A002_X905825_X5a8.ms.split',
    caltable = 'uid___A002_X905825_X5a8.ms.split.phase_int',
    field = '0~2', # J0241-0815,J0334-4008,J0215-0222
    solint = 'int',
    refant = 'DV04',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X905825_X5a8.ms.split.bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X905825_X5a8.ms.split.phase_int', msName='uid___A002_X905825_X5a8.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X905825_X5a8.ms.split.ampli_inf') 
  gaincal(vis = 'uid___A002_X905825_X5a8.ms.split',
    caltable = 'uid___A002_X905825_X5a8.ms.split.ampli_inf',
    field = '0~2', # J0241-0815,J0334-4008,J0215-0222
    solint = 'inf',
    refant = 'DV04',
    gaintype = 'T',
    calmode = 'a',
    gaintable = ['uid___A002_X905825_X5a8.ms.split.bandpass', 'uid___A002_X905825_X5a8.ms.split.phase_int'])
  
  if applyonly != True: es.checkCalTable('uid___A002_X905825_X5a8.ms.split.ampli_inf', msName='uid___A002_X905825_X5a8.ms.split', interactive=False) 
  
  os.system('rm -rf uid___A002_X905825_X5a8.ms.split.flux_inf') 
  os.system('rm -rf uid___A002_X905825_X5a8.ms.split.fluxscale') 
  mylogfile = casalog.logfile()
  casalog.setlogfile('uid___A002_X905825_X5a8.ms.split.fluxscale')
  
  fluxscaleDict = fluxscale(vis = 'uid___A002_X905825_X5a8.ms.split',
    caltable = 'uid___A002_X905825_X5a8.ms.split.ampli_inf',
    fluxtable = 'uid___A002_X905825_X5a8.ms.split.flux_inf',
    reference = '1') # J0334-4008
  
  casalog.setlogfile(mylogfile)
  
  if applyonly != True: es.fluxscale2(caltable = 'uid___A002_X905825_X5a8.ms.split.ampli_inf', removeOutliers=True, msName='uid___A002_X905825_X5a8.ms', writeToFile=True, preavg=10000)
  
  os.system('rm -rf uid___A002_X905825_X5a8.ms.split.phase_inf') 
  gaincal(vis = 'uid___A002_X905825_X5a8.ms.split',
    caltable = 'uid___A002_X905825_X5a8.ms.split.phase_inf',
    field = '0~2', # J0241-0815,J0334-4008,J0215-0222
    solint = 'inf',
    refant = 'DV04',
    gaintype = 'G',
    calmode = 'p',
    gaintable = 'uid___A002_X905825_X5a8.ms.split.bandpass')
  
  if applyonly != True: es.checkCalTable('uid___A002_X905825_X5a8.ms.split.phase_inf', msName='uid___A002_X905825_X5a8.ms.split', interactive=False) 
  

# Save flags before applycal
mystep = 16
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  
  flagmanager(vis = 'uid___A002_X905825_X5a8.ms.split',
    mode = 'save',
    versionname = 'BeforeApplycal')
  
  

# Application of the bandpass and gain cal tables
mystep = 17
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  for i in ['0', '1']: # J0241-0815,J0334-4008
    applycal(vis = 'uid___A002_X905825_X5a8.ms.split',
      field = str(i),
      gaintable = ['uid___A002_X905825_X5a8.ms.split.bandpass', 'uid___A002_X905825_X5a8.ms.split.phase_int', 'uid___A002_X905825_X5a8.ms.split.flux_inf'],
      gainfield = ['', i, i],
      interp = 'linear,linear',
      calwt = F,
      flagbackup = F)
  
  applycal(vis = 'uid___A002_X905825_X5a8.ms.split',
    field = '2,3', # mira
    gaintable = ['uid___A002_X905825_X5a8.ms.split.bandpass', 'uid___A002_X905825_X5a8.ms.split.phase_inf', 'uid___A002_X905825_X5a8.ms.split.flux_inf'],
    gainfield = ['', '2', '2'], # J0215-0222
    interp = 'linear,linear',
    calwt = F,
    flagbackup = F)
  

# Split out corrected column
mystep = 18
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf uid___A002_X905825_X5a8.ms.split.cal') 
  split(vis = 'uid___A002_X905825_X5a8.ms.split',
    outputvis = 'uid___A002_X905825_X5a8.ms.split.cal',
    datacolumn = 'corrected',
    keepflags = T)

  # Additional flags
  flagdata(vis='uid___A002_X905825_X5a8.ms.split.cal',
           mode='manual',
           spw='',
           antenna='DV07;DA49')
  
  # Split science target data. We perform limited channel averaging
  # of continuum spws to reduce file size, without losing any information.
  os.system('rm -rf uid___A002_X905825_X5a8.ms.split.cal.science') 
  split(vis = 'uid___A002_X905825_X5a8.ms.split.cal',
    outputvis = 'uid___A002_X905825_X5a8.ms.split.cal.science',
    datacolumn = 'data',
    field='3', spw='0,1,2,3,4,5,6', width=[16,16,16,1,1,1,1],
    keepflags = T)
